@@ -88,7 +88,8 @@ static bool switch_user = false;
 static uid_t uid;
 static gid_t gid;
 static char *user = NULL;
-static void daemon_become_new_user__(bool access_datapath);
+static void daemon_become_new_user__(bool access_datapath,
+                                     bool access_hardware_ports);
 
 static void check_already_running(void);
 static int lock_pidfile(FILE *, int command);
@@ -360,12 +361,15 @@ monitor_daemon(pid_t daemon_pid)
                                (unsigned long int) daemon_pid, status_msg);
 
         if (child_ready) {
+            char *log_file = vlog_get_log_file();
+            vlog_close_log_file();
             int error;
             do {
                 retval = waitpid(daemon_pid, &status, 0);
                 error = retval == -1 ? errno : 0;
             } while (error == EINTR);
-            vlog_reopen_log_file();
+            vlog_set_log_file(log_file);
+            free(log_file);
             if (error) {
                 VLOG_FATAL("waitpid failed (%s)", ovs_strerror(error));
             }
@@ -440,13 +444,13 @@ monitor_daemon(pid_t daemon_pid)
  * daemonize_complete()) or that it failed to start up (by exiting with a
  * nonzero exit code). */
 void
-daemonize_start(bool access_datapath)
+daemonize_start(bool access_datapath, bool access_hardware_ports)
 {
     assert_single_threaded();
     daemonize_fd = -1;
 
     if (switch_user) {
-        daemon_become_new_user__(access_datapath);
+        daemon_become_new_user__(access_datapath, access_hardware_ports);
         switch_user = false;
     }
 
@@ -804,7 +808,8 @@ daemon_become_new_user_unix(void)
 /* Linux specific implementation of daemon_become_new_user()
  * using libcap-ng.   */
 static void
-daemon_become_new_user_linux(bool access_datapath OVS_UNUSED)
+daemon_become_new_user_linux(bool access_datapath OVS_UNUSED,
+                             bool access_hardware_ports OVS_UNUSED)
 {
 #if defined __linux__ &&  HAVE_LIBCAPNG
     int ret;
@@ -824,6 +829,20 @@ daemon_become_new_user_linux(bool access_datapath OVS_UNUSED)
                 ret = capng_update(CAPNG_ADD, cap_sets, CAP_NET_ADMIN)
                       || capng_update(CAPNG_ADD, cap_sets, CAP_NET_RAW)
                       || capng_update(CAPNG_ADD, cap_sets, CAP_NET_BROADCAST);
+#ifdef DPDK_NETDEV
+                if (access_hardware_ports && !ret) {
+                    ret = capng_update(CAPNG_ADD, cap_sets, CAP_SYS_RAWIO);
+                    if (!ret) {
+                        VLOG_INFO("The Linux capability CAP_SYS_RAWIO "
+                                  "is enabled.");
+                    }
+                }
+#else
+                if (access_hardware_ports) {
+                    VLOG_WARN("No driver requires Linux capability "
+                              "CAP_SYS_RAWIO, disabling it.");
+                }
+#endif
             }
         } else {
             ret = -1;
@@ -851,7 +870,7 @@ daemon_become_new_user_linux(bool access_datapath OVS_UNUSED)
 }
 
 static void
-daemon_become_new_user__(bool access_datapath)
+daemon_become_new_user__(bool access_datapath, bool access_hardware_ports)
 {
     /* If vlog file has been created, change its owner to the non-root user
      * as specifed by the --user option.  */
@@ -859,7 +878,8 @@ daemon_become_new_user__(bool access_datapath)
 
     if (LINUX) {
         if (LIBCAPNG) {
-            daemon_become_new_user_linux(access_datapath);
+            daemon_become_new_user_linux(access_datapath,
+                                         access_hardware_ports);
         } else {
             VLOG_FATAL("%s: fail to downgrade user using libcap-ng. "
                        "(libcap-ng is not configured at compile time), "
@@ -874,11 +894,11 @@ daemon_become_new_user__(bool access_datapath)
  * However, there in case the user switch needs to be done
  * before daemonize_start(), the following API can be used.  */
 void
-daemon_become_new_user(bool access_datapath)
+daemon_become_new_user(bool access_datapath, bool access_hardware_ports)
 {
     assert_single_threaded();
     if (switch_user) {
-        daemon_become_new_user__(access_datapath);
+        daemon_become_new_user__(access_datapath, access_hardware_ports);
         /* daemonize_start() should not switch user again. */
         switch_user = false;
     }
@@ -950,7 +970,7 @@ enlarge_buffer(char **buf, size_t *sizep)
 void
 daemon_set_new_user(const char *user_spec)
 {
-    char *pos = strchr(user_spec, ':');
+    const char *pos = strchr(user_spec, ':');
     size_t init_bufsize, bufsize;
 
     init_bufsize = get_sysconf_buffer_size();
@@ -979,7 +999,7 @@ daemon_set_new_user(const char *user_spec)
         }
 
         if (e != 0) {
-            VLOG_FATAL("%s: Failed to retrive user %s's uid (%s), aborting.",
+            VLOG_FATAL("%s: Failed to retrieve user %s's uid (%s), aborting.",
                        pidfile, user, ovs_strerror(e));
         }
         if (res == NULL) {
@@ -994,7 +1014,7 @@ daemon_set_new_user(const char *user_spec)
         }
 
         if (e != 0) {
-            VLOG_FATAL("%s: Failed to retrive current user's name "
+            VLOG_FATAL("%s: Failed to retrieve current user's name "
                        "(%s), aborting.", pidfile, ovs_strerror(e));
         }
         user = xstrdup(pwd.pw_name);
@@ -1005,7 +1025,7 @@ daemon_set_new_user(const char *user_spec)
     free(buf);
 
     if (pos) {
-        char *grpstr = pos + 1;
+        const char *grpstr = pos + 1;
         grpstr += strspn(grpstr, " \t\r\n");
 
         if (*grpstr) {
