@@ -47,7 +47,7 @@ VLOG_DEFINE_THIS_MODULE(dpdk);
 static FILE *log_stream = NULL;       /* Stream for DPDK log redirection */
 
 /* Indicates successful initialization of DPDK. */
-static atomic_bool dpdk_initialized = ATOMIC_VAR_INIT(false);
+static atomic_bool dpdk_initialized = false;
 
 static bool
 args_contains(const struct svec *args, const char *value)
@@ -123,7 +123,7 @@ construct_dpdk_mutex_options(const struct smap *ovs_other_config,
     int i;
     for (i = 0; i < ARRAY_SIZE(excl_opts); ++i) {
         int found_opts = 0, scan, found_pos = -1;
-        const char *found_value;
+        const char *found_value = NULL;
         struct dpdk_exclusive_options_map *popt = &excl_opts[i];
 
         for (scan = 0; scan < MAX_DPDK_EXCL_OPTS
@@ -364,8 +364,8 @@ dpdk_init__(const struct smap *ovs_other_config)
              * thread affintity - default to core #0 */
             VLOG_ERR("Thread getaffinity failed. Using core #0");
         }
-        svec_add(&args, "-l");
-        svec_add_nocopy(&args, xasprintf("%d", cpu));
+        svec_add(&args, "--lcores");
+        svec_add_nocopy(&args, xasprintf("0@%d", cpu));
     }
 
     svec_terminate(&args);
