@@ -672,8 +672,6 @@ netdev_afxdp_get_config(const struct netdev *netdev, struct smap *args)
     ovs_mutex_lock(&dev->mutex);
     smap_add_format(args, "n_rxq", "%d", netdev->n_rxq);
     smap_add_format(args, "xdp-mode", "%s", xdp_modes[dev->xdp_mode].name);
-    smap_add_format(args, "xdp-mode-in-use", "%s",
-                    xdp_modes[dev->xdp_mode_in_use].name);
     smap_add_format(args, "use-need-wakeup", "%s",
                     dev->use_need_wakeup ? "true" : "false");
     ovs_mutex_unlock(&dev->mutex);
@@ -896,7 +894,7 @@ netdev_afxdp_rxq_recv(struct netdev_rxq *rxq_, struct dp_packet_batch *batch,
 
 #if __GNUC__ >= 11 && !__clang__
         /* GCC 11+ generates a false-positive warning about free() being
-         * called on DPBUF_AFXDP packet, but it is an imposisible code path.
+         * called on DPBUF_AFXDP packet, but it is an impossible code path.
          * Disabling a warning to avoid build failures.
          * https://gcc.gnu.org/bugzilla/show_bug.cgi?id=108187 */
 #pragma GCC diagnostic push
@@ -1195,18 +1193,18 @@ libbpf_print(enum libbpf_print_level level,
     return 0;
 }
 
-int netdev_afxdp_init(void)
-{
-    libbpf_set_print(libbpf_print);
-    return 0;
-}
-
 int
 netdev_afxdp_construct(struct netdev *netdev)
 {
+    static struct ovsthread_once once = OVSTHREAD_ONCE_INITIALIZER;
     struct netdev_linux *dev = netdev_linux_cast(netdev);
     int ret;
 
+    if (ovsthread_once_start(&once)) {
+        libbpf_set_print(libbpf_print);
+        ovsthread_once_done(&once);
+    }
+
     /* Configure common netdev-linux first. */
     ret = netdev_linux_construct(netdev);
     if (ret) {
@@ -1367,3 +1365,22 @@ netdev_afxdp_get_stats(const struct netdev *netdev,
 
     return error;
 }
+
+int
+netdev_afxdp_get_status(const struct netdev *netdev, struct smap *args)
+{
+    int error = netdev_linux_get_status(netdev, args);
+
+    if (error) {
+        return error;
+    }
+
+    struct netdev_linux *dev = netdev_linux_cast(netdev);
+
+    ovs_mutex_lock(&dev->mutex);
+    smap_add_format(args, "xdp-mode", "%s",
+                    xdp_modes[dev->xdp_mode_in_use].name);
+    ovs_mutex_unlock(&dev->mutex);
+
+    return error;
+}
