@@ -28,9 +28,9 @@ struct stream;
 struct pstream;
 struct sockaddr_storage;
 
-int new_fd_stream(const char *name, int fd, int connect_status,
+int new_fd_stream(char *name, int fd, int connect_status,
                   int fd_type, struct stream **streamp);
-int new_fd_pstream(const char *name, int fd,
+int new_fd_pstream(char *name, int fd,
                    int (*accept_cb)(int fd, const struct sockaddr_storage *ss,
                                     size_t ss_len, struct stream **),
                    char *unlink_path,
