@@ -125,6 +125,8 @@ struct bond {
     uint32_t basis;             /* Basis for flow hash function. */
     bool use_lb_output_action;  /* Use lb_output action to avoid recirculation.
                                    Applicable only for Balance TCP mode. */
+    bool all_members_active;    /* Accept multicast packets on secondary
+                                   members of a non-LACP Balance SLB bond. */
     char *primary;              /* Name of the primary member. */
 
     /* SLB specific bonding info. */
@@ -240,8 +242,10 @@ bond_create(const struct bond_settings *s, struct ofproto_dpif *ofproto)
     bond = xzalloc(sizeof *bond);
     bond->ofproto = ofproto;
     hmap_init(&bond->members);
-    ovs_list_init(&bond->enabled_members);
     ovs_mutex_init(&bond->mutex);
+    ovs_mutex_lock(&bond->mutex);
+    ovs_list_init(&bond->enabled_members);
+    ovs_mutex_unlock(&bond->mutex);
     ovs_refcount_init(&bond->ref_cnt);
     hmap_init(&bond->pr_rule_ops);
 
@@ -482,6 +486,9 @@ bond_reconfigure(struct bond *bond, const struct bond_settings *s)
         revalidate = true;
     }
 
+    bond->all_members_active = (bond->balance == BM_SLB)
+                               ? s->all_members_active : false;
+
     if (bond->balance != BM_AB) {
         if (!bond->recirc_id) {
             bond->recirc_id = recirc_alloc_id(bond->ofproto);
@@ -911,7 +918,7 @@ bond_check_admissibility(struct bond *bond, const void *member_,
 
     /* Drop all multicast packets on inactive members. */
     if (eth_addr_is_multicast(eth_dst)) {
-        if (bond->active_member != member) {
+        if (bond->active_member != member && !bond->all_members_active) {
             goto out;
         }
     }
@@ -1056,7 +1063,7 @@ bond_may_recirc(const struct bond *bond)
 }
 
 static void
-bond_update_post_recirc_rules__(struct bond* bond, bool force)
+bond_update_post_recirc_rules__(struct bond* bond, const bool force)
     OVS_REQ_WRLOCK(rwlock)
 {
    struct bond_entry *e;
@@ -1526,6 +1533,11 @@ bond_print_details(struct ds *ds, const struct bond *bond)
                   use_lb_output_action ? "enabled" : "disabled",
                   use_lb_output_action ? recirc_id : -1);
 
+    if (bond->balance == BM_SLB) {
+        ds_put_format(ds, "all members active: %s\n",
+                      bond->all_members_active ? "true" : "false");
+    }
+
     ds_put_format(ds, "updelay: %d ms\n", bond->updelay);
     ds_put_format(ds, "downdelay: %d ms\n", bond->downdelay);
 
